#!/bin/bash
# This script creates an Aegir platform, from the download up to the front-end.

#****************************************
# General variables
#****************************************

# The name of this file.
kplatforms_this_file=$(basename $0)

# The full path of this file.
kplatforms_this_path=$(dirname $(pwd)/$0)

# Today's date and time.
kplatforms_date="$(date +%Y).$(date +%m).$(date +%d)"
kplatforms_time="$(date +%H).$(date +%M).$(date +%S)"

# Requirements
kplatforms_req_user="aegir"
kplatforms_req_drush_version="4.5"
kplatforms_req_drush_make_version="2.3"

# Error and warning detection.
kplatforms_error=0
kplatforms_warning=0

#****************************************
# Arguments
#****************************************

# Processing arguments and defining variables.
while getopts ":a:c:d:fh:i:n:p:s:v:" opt; do
  case $opt in

    a) # -a htpasswd_file
      kplatforms_htpasswd_file="$OPTARG"
      ;;

    c) # -c config_file
      kplatforms_config_file="$OPTARG"

      # Checking if we can read the file and include it.
      if [ ! -r "$kplatforms_config_file" ]; then
        echo "ERROR: The configuration file '$kplatforms_config_file' cannot be read."
        kplatforms_error=`expr $kplatforms_error + 1`
      else
        . $kplatforms_config_file
      fi

      ;;

    d) # -d drush_command
      kplatforms_drush_command="$OPTARG"
      ;;

    f) # force_complete
      kplatforms_force_complete="true"
      ;;

    h) # -h hostmaster_instance_name
      kplatforms_hostmaster_instance_name="$OPTARG"
      ;;

    i) # -i inclusions
      kplatforms_inclusions="$OPTARG"
      ;;

    n) # -n platform_name
      kplatforms_platform_name="$OPTARG"
      ;;

    p) # -p platforms_path
      kplatforms_platforms_path="$OPTARG"
      ;;

    s) # -s platform_suffix
      kplatforms_platform_suffix="$OPTARG"
      ;;

    v) # -v drupal_version
      kplatforms_drupal_version="$OPTARG"

      # Checking if Drupal version is supported.
      if [ $kplatforms_drupal_version != 7 -a $kplatforms_drupal_version != 6 -a $kplatforms_drupal_version != 5 ] ; then
        echo "ERROR: Version '$kplatforms_drupal_version' is not supported."
        echo "Version must be '7', '6' or '5'. Default is '7'."
        kplatforms_error=`expr $kplatforms_error + 1`
      fi

      ;;

    \?) # Invalid option.
      echo "ERROR: Invalid option: -$OPTARG"
      kplatforms_error=`expr $kplatforms_error + 1`
      ;;

    :) # Missing argument.
      echo "ERROR: Option -$OPTARG requires an argument."
      kplatforms_error=`expr $kplatforms_error + 1`
      ;;

  esac
done

# If error, echo usage and exit.
if [ $kplatforms_error -ne 0 ]; then
  echo "Usage:"
  echo ""
  echo "$kplatforms_this_file [-a htpasswd_file] [-c config_file] [-d drush_command] [-f] [-h hostmaster_instance_name] [-i inclusions] [-n platform_name] [-p platforms_path] [-s platform_suffix] [-v drupal_version]"
  echo ""
  echo "Where 'inclusions' are 'inclusion[,inclusion]...'"
  exit $kplatforms_error
fi

#****************************************
# Default variables
#****************************************

# If variable is not already defined neither by argument, config file nor shell variable, then...

# First of all, reading the default configuration file if no one has been defined.
if [ -z $kplatforms_config_file ]; then
  kplatforms_config_file="/var/aegir/config/kplatforms.conf"
  if [ -r "$kplatforms_config_file" ]; then
    . $kplatforms_config_file
  fi
fi

# Drupal core major version number.
if [ -z $kplatforms_drupal_version ]; then
  kplatforms_drupal_version=7
fi
# Platform name.
if [ -z $kplatforms_platform_name ]; then
  kplatforms_platform_name="drupal-$kplatforms_drupal_version"
fi

# Drush command.
if [ -z $kplatforms_drush_command ]; then
  kplatforms_drush_command="drush"
fi

# Hosmaster instance name.
# To use like "drush @hostmaster ...".
if [ -z $kplatforms_hostmaster_instance_name ]; then
  kplatforms_hostmaster_instance_name="hostmaster"
fi

# Inclusions
if [ -z $kplatforms_inclusions ]; then
  kplatforms_inclusions="modules themes libraries"
fi

# Platforms path, directory where to create platforms.
if [ -z $kplatforms_platforms_path ]; then
  kplatforms_platforms_path="/var/aegir/platforms"
fi

# Temporary directory where to build the stub makefile.
if [ -z $kplatforms_tmpdir ]; then
  kplatforms_tmpdir="/tmp"
fi

#****************************************
# Checking requirements
#****************************************

# User

kplatforms_user=$(id -nu)
echo "Current user = '$kplatforms_user'"

if [ "$kplatforms_user" != "$kplatforms_req_user" ] ; then
  echo "ERROR: You must run '$kplatforms_this_file' as user '$kplatforms_req_user'."
  exit 1
fi

# Drush version

kplatforms_drush_version=$($kplatforms_drush_command --version | awk '{print $NF}')
echo "Current 'drush' version = '$kplatforms_drush_version'"

if [ $(echo "$kplatforms_drush_version < $kplatforms_req_drush_version" | bc) -eq 1 ]; then
  echo "ERROR: 'Drush' version must be '$kplatforms_req_drush_version' or higher."
  exit 1
fi

# Drush make version

kplatforms_drush_make_version=$($kplatforms_drush_command make --version | awk '{print $NF}')
echo "Current 'drush make' version = '$kplatforms_drush_make_version'"

if [ $(echo "$kplatforms_drush_make_version < $kplatforms_req_drush_make_version" | bc) -eq 1 ]; then
  echo "ERROR: 'Drush make' version must be '$kplatforms_req_drush_make_version' or higher."
  exit 1
fi

#****************************************
# Building platform information
#****************************************

# The name of the platform directory without any path.
if [ "$kplatforms_platform_suffix" = "" ]; then
  kplatforms_platform_directory="$kplatforms_platform_name-build-$kplatforms_date"
else
  kplatforms_platform_directory="$kplatforms_platform_name-build-$kplatforms_date-$kplatforms_platform_suffix"
fi
echo "Platform name = '$kplatforms_platform_directory'"

# Absolute path to the platform.
kplatforms_platform_path="$kplatforms_platforms_path/$kplatforms_platform_directory"
echo "Platform path = '$kplatforms_platform_path'"

#****************************************
# Building stub makefile
#****************************************

# Directory where to find the makefiles to include.
kplatforms_includir="$kplatforms_this_path/includes/$kplatforms_drupal_version"
echo "Includes directory = '$kplatforms_includir'"

# The stub makefile with its full path.
kplatforms_makefile="$kplatforms_tmpdir/kplatforms-stub_$kplatforms_platform_directory-$kplatforms_time.make"
echo "Stub makefile = '$kplatforms_makefile'"

# Including specifications while creating the stub makefile.
echo "api = 2" >$kplatforms_makefile
echo "core = $kplatforms_drupal_version.x" >>$kplatforms_makefile

#****************************************
# Including makefiles in the stub
#****************************************

# Including core makefile.
echo "includes[core] = $kplatforms_includir/core.make" >>$kplatforms_makefile

# Replacing upper case by lower case.
kplatforms_inclusions=$(echo $kplatforms_inclusions | tr '[:upper:]' '[:lower:]')

# Replacing comas by space if a coma separated list is given.
kplatforms_inclusions=$(echo $kplatforms_inclusions | sed 's/,/ /g')

echo "Inclusions = '$kplatforms_inclusions'"

# Processing inclusions.
for kplatforms_inclusion in $kplatforms_inclusions; do

  # The makefile with its full path.
  kplatforms_inclusion_makefile="$kplatforms_includir/$kplatforms_inclusion.make"
  echo "Including makefile '$kplatforms_inclusion_makefile'"

  # Checking if we can read the makefile.
  if [ ! -r "$kplatforms_inclusion_makefile" ]; then
    echo "ERROR: The makefile '$kplatforms_inclusion_makefile' cannot be read."
    rm $kplatforms_makefile
    exit 1
  fi

  # Including the makefile.
  echo "includes[$kplatforms_inclusion] = $kplatforms_inclusion_makefile" >>$kplatforms_makefile

done

#****************************************
# Drush commands
#****************************************

# Using "drush --force-complete".
kplatforms_force_complete=$(echo $kplatforms_force_complete | tr '[:upper:]' '[:lower:]')   # Replacing upper case by lower case.
if [ "$kplatforms_force_complete" = "true" ]; then
  kplatforms_drush_command="$kplatforms_drush_command --force-complete"
fi
echo "Drush command = '$kplatforms_drush_command'"

# Creating platform.
$kplatforms_drush_command make $kplatforms_makefile $kplatforms_platform_path && \
  $kplatforms_drush_command --root=$kplatforms_platform_path provision-save @platform_$kplatforms_platform_directory --context_type=platform --makefile=$kplatforms_makefile && \
  $kplatforms_drush_command @$kplatforms_hostmaster_instance_name hosting-import @platform_$kplatforms_platform_directory

# Test exit code.
if [ $? -eq 0 ]; then
  echo "OK: Platform created successfully in '$kplatforms_platform_path'."
else
  echo "ERROR: Platform not created successfully."
  exit 1
fi

#****************************************
# Access restricted by users (htauth)
#****************************************

if [ "$kplatforms_htpasswd_file" != "" ]; then

  # Checking if we can read the htpasswd file.
  if [ ! -r "$kplatforms_htpasswd_file" ]; then
    echo "WARNING: The htpasswd file cannot be read."
    echo "Use: 'htpasswd -b -c $kplatforms_htpasswd_file username password'."
    kplatforms_warning=`expr $kplatforms_warning + 1`
  fi

  # Defining variables.
  kplatforms_htaccess_file="$kplatforms_platform_path/.htaccess"
  kplatforms_htaccess_file_head="$kplatforms_platform_path/.htaccess.head"
  kplatforms_htaccess_file_tmp="$kplatforms_platform_path/.htaccess.tmp"

  # Editing htaccess file.
  echo "# Access restricted by users" >$kplatforms_htaccess_file_head && \
    echo "AuthUserFile $kplatforms_htpasswd_file" >>$kplatforms_htaccess_file_head && \
    echo "AuthName 'Authorized people only'" >>$kplatforms_htaccess_file_head && \
    echo "AuthType Basic" >>$kplatforms_htaccess_file_head && \
    echo "require valid-user" >>$kplatforms_htaccess_file_head && \
    echo "" >>$kplatforms_htaccess_file_head && \
    cat $kplatforms_htaccess_file_head $kplatforms_htaccess_file >$kplatforms_htaccess_file_tmp && \
    mv $kplatforms_htaccess_file_tmp $kplatforms_htaccess_file && \
    rm $kplatforms_htaccess_file_head

  # Test exit code.
  if [ $? -eq 0 ]; then
    echo "OK: Sites on this platform restricted successfully with '$kplatforms_htpasswd_file'."
  else
    echo "ERROR: Sites on this platform not restricted successfully."
    exit 1
  fi

fi

#****************************************
# Platform improvements
#****************************************

# Favicon, for flavour.
cp $kplatforms_this_path/favicon.ico $kplatforms_platform_path/
if [ $? -eq 0 ]; then
  echo "OK: Favicon file copied successfully."
else
  echo "WARNING: Favicon file not copied successfully."
  kplatforms_warning=`expr $kplatforms_warning + 1`
fi

# Making changes as per the Drupal core version.
if [ $kplatforms_drupal_version -eq 6 ]; then

  # Making changes as per the inclusions.
  for kplatforms_inclusion in $kplatforms_inclusions; do
    case $kplatforms_inclusion in

      libraries)

        # TCPDF library for the Drupal contrib module "Printer, e-mail and PDF versions" (print).
        # Directories non-writable permissions.

          # Web server group:
          # Use www-data for Debian and Ubuntu.
          # Use apache for RedHat and CentOS.
          kplatforms_webserver_grp="www-data"

          # TODO add test for RedHat
          if [ -x /usr/bin/lsb_release ]; then
            kplatforms_os=$(lsb_release -i | cut -f 2)
            if [ "$kplatforms_os" = "CentOS" ]; then
              kplatforms_webserver_grp=apache
            fi
          fi

          # Modifying permissions and group.
          chmod g+w $kplatforms_platform_path/sites/all/libraries/tcpdf/cache $kplatforms_platform_path/sites/all/libraries/tcpdf/images && \
            chgrp $kplatforms_webserver_grp $kplatforms_platform_path/sites/all/libraries/tcpdf/cache $kplatforms_platform_path/sites/all/libraries/tcpdf/images

          # Test exit code.
          if [ $? -eq 0 ]; then
            echo "OK: TCPDF chmod and chgrp."
          else
            echo "WARNING: TCPDF chmod and chgrp not done successfully."
            kplatforms_warning=`expr $kplatforms_warning + 1`
          fi

      ;;   # End of libraries.

    esac
  done     # End of inclusions.

fi         # End of Drupal 6.

#****************************************
# Exiting
#****************************************

if [ $kplatforms_warning -eq 0 ]; then
  echo "OK: '$kplatforms_this_file' terminated successfully."
  exit 0
elif [ $kplatforms_warning -eq 1 ]; then
  echo "WARNING: '$kplatforms_this_file' terminated with $kplatforms_warning warning."
  exit $kplatforms_warning
else
  echo "WARNING: '$kplatforms_this_file' terminated with $kplatforms_warning warnings."
  exit $kplatforms_warning
fi

#****************************************
# End
#****************************************
